from deepclas4bio.listMethods import *
from deepclas4bio.predict import *
from deepclas4bio.evaluate import *

__all__=[predict,predictBatch,listFrameworks,listModels,listMeasures,listReadDatasets,evaluate]

predict.__module__='deepclas4bio'
predictBatch.__module__='deepclas4bio'
listFrameworks.__module__='deepclas4bio'
listModels.__module__='deepclas4bio'
listMeasures.__module__='deepclas4bio'
listReadDatasets.__module__='deepclas4bio'
evaluate.__module__='deepclas4bio'
